<?php

if ( ! defined( 'ABSPATH' ) ) exit;

if (!class_exists('FastMailchimp_core')) {

    class FastMailchimp_core {

      private $mailchimp_url;
      private $fast_mailchimp_apikey;

      public function __construct(){
        $this->mailchimp_url = 'https://us6.api.mailchimp.com/3.0/';
        $this->fast_mailchimp_apikey();
        add_action('admin_notices', array($this, 'fast_mailchimp_admin_notice__error'));
        add_filter('ff_settings', array($this, 'fast_mailchimp_settings_html'), 83, 1 );
        add_filter('ff_settings_data', array($this, 'fast_process_mailchimp_data'), 10, 1);
        add_filter('FM_AR_select_options_addons', array($this, 'fast_mailchimp_AR_select_options'), 83, 2 );
        add_filter('FM_AR_options_HTML_addons', array($this, 'fast_mailchimp_AR_options_HTML'), 83, 2 );
        add_action('FM_add_to_AR_addons', array($this, 'fast_mailchimp_add_to_AR'), 83, 2 );
      }

      public function fast_mailchimp_apikey(){
        $mailchimp_db = $this->fast_mailchimp_settings_db('Mailchimp');
        $mailchimp_options = empty( $mailchimp_db->settings_data ) ? array() : unserialize( $mailchimp_db->settings_data );
        $this->fast_mailchimp_apikey = empty( $mailchimp_options['fast_mailchimp_apikey'] ) ? '' : $mailchimp_options['fast_mailchimp_apikey'];
      }

      public function fast_mailchimp_admin_notice__error(){
        if( is_plugin_active( 'fast-mailchimp/fast-mailchimp.php' ) ) {
          $class = 'notice notice-error';
          $message = 'Please input API keys for <a href="'.admin_url("admin.php?page=fast-flow-settings").'">Mailchimp.</a>';
          if(!empty($this->fast_mailchimp_apikey)){
            return true;
          }else{
            printf( '<div class="%1$s is-dismissible"><p>%2$s</p></div>', esc_attr( $class ), ($message) );
          }
        }else{
          return true;
        }
      }

      public function fast_mailchimp_settings_html($html){
        $settings_html = '<h1><strong>Mailchimp</strong></h1>';
        $settings_html .= '<div class="item-tab-box">';
        $settings_html .= '<table cellspacing="10" width="100%">
                          <tr><td width="30%">'.__("API Key").':</td><td width="70%"><input type="text" id="fast_mailchimp_apikey" style="width: 390px;" name="fast_mailchimp_apikey" value="' . $this->fast_mailchimp_apikey . '" /></td></tr>';
        $settings_html .=' </table>';
        $settings_html .= '</div>';
        return $html.$settings_html;
      }

      public function fast_process_mailchimp_data($data){
        global $wpdb;
        $data_arr = array();
        $data_arr['fast_mailchimp_apikey'] = empty( $data['fast_mailchimp_apikey'] ) ? '' : sanitize_text_field( $data['fast_mailchimp_apikey'] );
        $data_ser = serialize($data_arr);
        $count = $wpdb->get_var( "SELECT COUNT(ID) FROM {$wpdb->prefix}fastflow_settings
                                                                            WHERE settings_for='Mailchimp'" );

          if( $count == 1 ) {
              $wpdb->update(
                  $wpdb->prefix . 'fastflow_settings',
                  array( 'settings_data' => $data_ser, 'extra_data' => '' ),
                  array( 'settings_for' => 'Mailchimp' ),
                  array( '%s', '%s' ),
                  array( '%s' )
                );
          } else {
              $wpdb->insert(
                  $wpdb->prefix . 'fastflow_settings',
                  array( 'settings_for' => 'Mailchimp', 'settings_data' => $data_ser, 'extra_data' => '' ),
                  array( '%s', '%s', '%s' ) );
          }
          return $data;
      }

      public function fast_mailchimp_settings_db($for = ''){
        if( !empty( $for ) && $for !== '' ) {
            global $wpdb;
            $data = $wpdb->get_results( $wpdb->prepare( "SELECT * FROM {$wpdb->prefix}fastflow_settings WHERE settings_for=%s", $for ) );
            if( count($data) >= 1 ) {
                return $data[0];
            } else return false;
        }
        return false;
      }

      public function fast_mailchimp_AR_select_options($value, $selected_opts){
        $selected_attr = ( $selected_opts == 12 ) ? "selected='selected'" : "";
        $fsn_opts = "<option value='12'{$selected_attr}>Mailchimp</option>";
        $return_val = $fsn_opts . $value;
        return $return_val;
      }

      public function fast_mailchimp_AR_options_HTML( $value, $selected_opts ){
          $mldisp = ( $selected_opts == 12 ) ? "block" : "none";
          $mailchimp_list_id = sanitize_text_field($_POST['mailchimp_list_id']);
          $lists = $this->get_fast_mailchimp_lists();
          $return_html = $value . "<div style='display:$mldisp' id='arbox12' class='arcontentbox'><table cellspacing='10'><tr><td style='width: 140px;'>Choose List:</td><td>";
          $return_html .= "<select id='mailchimp_list_id' style='width: 200px;' name='mailchimp_list_id'>";
          if(!empty($lists)){
            foreach($lists as $list){
              $selected = ($list->id == $mailchimp_list_id)?"selected='selected'":"";
              $return_html .= "<option value='".$list->id."' ".$selected.">".$list->name."</option>";
            }
          }
          $return_html .= "</select>";
          $return_html .= "</td></tr></table></div>";

          return $return_html;
      }

      public function get_fast_mailchimp_lists(){
        $response = wp_remote_get( $this->mailchimp_url.'/lists', array(
          'method' => 'GET',
          'timeout' => 45,
          'headers' => array(
            'Content-type' => 'application/json',
            'Authorization' => 'Bearer '.$this->fast_mailchimp_apikey
           )
          )
        );

        if ( is_wp_error( $response ) ) {
           $error_message = $response->get_error_message();
           return new WP_Error( $response->get_error_code(), $error_message);
        } else {
          if(wp_remote_retrieve_response_code($response) == 200){
            $body = wp_remote_retrieve_body($response);
            $bodyArr = json_decode($body);
            return $bodyArr->lists;
          }else{
            return false;
          }
        }
      }

      public function fast_mailchimp_add_to_AR($user_id, $prod_id){
        global $wpdb;
        $this->user_id = $user_id;
        $this->prod_id = $prod_id;
        $pdata = $wpdb->get_row($wpdb->prepare("SELECT * FROM {$wpdb->prefix}wpbn_products WHERE id=%d",$this->prod_id));
        $arservice = $pdata->arservice;
        if( $arservice != 12 ) { return; }
        $aroptions = unserialize($pdata->aroptions);
        $mailchimp_list_id = $aroptions['mailchimp_list_id'];
        if($this->fast_mailchimp_apikey == ''){
          error_log( "Empty Mailchimp Api key" );
          return;
        }
        $email = get_user_by('id', $this->user_id)->user_email;
        $first_name = get_user_meta($this->user_id, 'first_name', true);
        $last_name = get_user_meta($this->user_id, 'last_name', true);
        $full_name = $first_name . " " . $last_name;
        $mailchimp_name = trim( $full_name );
        if ( empty( $mailchimp_name ) || $mailchimp_name == "" ){ $mailchimp_name = $email;}
        $user_fast_tags = wp_get_object_terms( $this->user_id, 'fast_tag', array('order' => 'ASC'));

        $mailchimp_member = $this->fast_mailchimp_add_member_to_list($mailchimp_list_id, $mailchimp_name, $email, $user_fast_tags);
      }

      public function fast_mailchimp_add_member_to_list($mailchimp_list_id, $mailchimp_name, $email, $user_fast_tags){
        $tags = [];
        if($user_fast_tags){
          foreach($user_fast_tags as $tag){
            $tags[] = $tag->name;
          }
        }
        $response = wp_remote_get( $this->mailchimp_url.'/lists/'.$mailchimp_list_id.'/members', array(
          'method' => 'POST',
          'timeout' => 45,
          'headers' => array(
            'Content-type' => 'application/json',
            'Authorization' => 'Bearer '.$this->fast_mailchimp_apikey
           ),
           'body' => json_encode([
              'email_address' => $email,
              'status' => 'subscribed',
              'tags' => $tags,
              'full_name' => $mailchimp_name
           ])
         )
        );

        if ( is_wp_error( $response ) ) {
           $error_message = $response->get_error_message();
           return new WP_Error( $response->get_error_code(), $error_message);
        } else {
          if(wp_remote_retrieve_response_code($response) == 200){
            $body = wp_remote_retrieve_body($response);
            $log_var = "<pre>" . print_r($body, true) . "</pre>";
            error_log("Mailchimp AR event log".$log_var);
            $bodyArr = json_decode($body);
            return $bodyArr;
          }else{
            return false;
          }
        }
      }

    }

}
